/*
panorama viewer
by Martin Pearman April 2006
*/

import java.awt.*;
import java.applet.*;
// These classes are for Url's.
import java.net.*;
import java.awt.event.*;
import javax.swing.*;

public class panViewer extends Applet implements MouseListener, MouseMotionListener, Runnable
{
	Image vrImage;
	URL base;

	int imageWidth;
	int imageHeight;
	int displayWidth;
	int displayHeight;
	String imageName;
	int fullPanorama;
	boolean panLeftToggle;
	boolean panRightToggle;
	int panPosition;
	int panDelay;
	int panMotion;
	Thread timer;
		
	// This object will allow you to control image loading
	MediaTracker mt;

public void init() 
{
	//	applet takes three parameters
	//	imageName - name of panoramic image in same directory as panViewer.class
	//	panWrap - a value of 1 indicates a full 360 degree panoramic image
	//	panPos - set the initial view position
	displayWidth=Integer.parseInt(getParameter("width"));
	displayHeight=Integer.parseInt(getParameter("height"));
	imageName=getParameter("imageName");
	fullPanorama=Integer.parseInt(getParameter("panWrap"));
	panPosition=Integer.parseInt(getParameter("panPos"));
	panLeftToggle=false;
	panRightToggle=false;
	panDelay=displayWidth/2;
	panMotion=1;

	// initialize the MediaTracker
	mt = new MediaTracker(this);

	// The try-catch is necassary when the URL isn't valid
	// Of course this one is valid, since it is generated by
	// Java itself.
	try
	{
		// getDocumentbase gets the applet path.
		base = getDocumentBase();
	}
	catch (Exception e) {}

	vrImage = getImage(base,imageName);
	// tell the MediaTracker to keep an eye on this image, and give it ID 1;
	mt.addImage(vrImage,1);

	// now tell the mediaTracker to stop the applet execution
	// until the image is fully loaded.
	// must be in a try catch block.
	try
	{
		mt.waitForAll();
	}
	catch (InterruptedException  e)
	{}
	// when the applet gets here then the image is loaded.
	imageWidth=vrImage.getWidth(this);
	imageHeight=vrImage.getHeight(this);
	if ((panPosition<(-imageWidth)) || (panPosition>imageWidth)) panPosition=0;
		
	addMouseListener(this);	
	addMouseMotionListener(this);
}

	public void paint(Graphics g) 
	{
		displayRedraw(g);
	}
	
	public void update(Graphics g)
	{
		displayRedraw(g);	
	}
	
	public void displayRedraw(Graphics g)
	{
		g.drawImage(vrImage,panPosition,0,this);
		if (fullPanorama==1)
		{
			if (panPosition<(imageWidth-displayWidth)) g.drawImage(vrImage,panPosition+imageWidth,0,this);
			if (panPosition>0) g.drawImage(vrImage,panPosition-imageWidth,0,this);
		}
	}
	
	public void mouseEntered(MouseEvent event)
	{
		//	enable panning - set panning settings
		setPanning(event);
		start();
	}
	
	public void mouseMoved(MouseEvent event)
	{
		//	update panning settings
		setPanning(event);
	}
	
	public void mouseExited(MouseEvent event)
	{
		// stop panning on mouse exit
		stop();
	}

	public void mouseReleased(MouseEvent event)
	{
		// null code
	}
	
	public void mousePressed(MouseEvent event)
	{
		// null code
	} 
	
	public void mouseClicked(MouseEvent event)
	{
		// null code
	}

	public void mouseDragged(MouseEvent event)
	{
		// null code
	}

	public void start()
	{
		if (timer==null)
		{
			timer=new Thread(this);
			timer.start();
		}
	}

	public void stop()
	{
		if (timer!=null) timer=null;
	}


	public void run()
	{
		while (timer==Thread.currentThread())
		{
			if (panLeftToggle)
			{
				panLeft();
				repaint();
			}
			if (panRightToggle)
			{
				panRight();
				repaint();
			}
			try
			{
				Thread.sleep(panDelay);
			}
			catch(InterruptedException e) {}
		}
	}

	public void panLeft()
	{
		if (fullPanorama==1)
		{
			panPosition+=panMotion;
			if (panPosition>displayWidth) panPosition=(-imageWidth)+displayWidth;
		}
		if (fullPanorama!=1)
		{
			if (panPosition<0) panPosition+=panMotion;
		}
	}

	public void panRight()
	{
		if (fullPanorama==1)
		{
			panPosition-=panMotion;
			if (panPosition<(-imageWidth)) panPosition=0;
		}
		if (fullPanorama!=1)
		{
			if (panPosition>(displayWidth-imageWidth)) panPosition-=panMotion;
		}
	}

	public void setPanning(MouseEvent event)
	{
		int xPos=event.getX();
		if (xPos<(displayWidth/2))
		{
			// pan left
			panLeftToggle=true;
			panRightToggle=false;
			if (xPos>0) panDelay=xPos;
			setCursor(Cursor.getPredefinedCursor(Cursor.E_RESIZE_CURSOR));
		}
		else
		{
			// pan right
			panRightToggle=true;
			panLeftToggle=false;
			if (xPos<displayWidth) panDelay=displayWidth-xPos;
			setCursor(Cursor.getPredefinedCursor(Cursor.W_RESIZE_CURSOR));
		}
	}
	
}